package SimpleAuth;

use strict;
use MT::App;
use MT::Plugin;
use MT::PluginData;
use HTTP::Date;

@SimpleAuth::ISA = qw(MT::App);

@MT::Plugin::SimpleAuth::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::SimpleAuth({
    id => 'SimpleAuth',
    key => 'simpleauth',
    name => 'SimpleAuth',
});

sub init_request {
	my $app = shift;
	$app->SUPER::init_request(@_);
	$app->add_methods( auth => \&_auth );
	$app->{default_mode} = 'unprotected';
	$app->{requires_login} = 1 unless $app->mode eq 'unprotected';
	$app->add_methods( unprotected => \&_unprotected );
	$app->add_methods( login => \&_login );
	$app;
}

############################################################################## 

#Setting

my $data = MT::PluginData->load({ plugin => 'simpleauth',
                                  key=> 'configuration' });
my $datas = $data->data;
my $base = $datas->{base};
my $suffix = $datas->{suffix};
my $auth_dirs = $datas->{auth_dirs};
my $indexes = $datas->{indexes};
my $error_file = $datas->{error_file};

##############################################################################

sub _login {
	my $app = shift;
	my $q = $app->param;
	my $return_url = $q->param('return_url');
    $app->redirect($return_url);
1;
}

sub _unprotected {
	my $app = shift;
	my $q = $app->param;
	my $return_url = $q->param('return_url');
    my $path = $app->param('path') || $ENV{"REDIRECT_URL"};
    $path =~ s/\.\.//g;
    my $org = $path;
    $path = $base.$path;
    my $auth_flg;
    my @dirs = split("\n", $auth_dirs);
    foreach my $ck (@dirs) {
        if ($path =~ /^$ck/) {
            $auth_flg = 1;
            last;
        }
    }
    unless ($auth_flg) {
        $app->set_header('Status','404 File Not Found');
        $app->send_http_header();
        if (($error_file) && (-f $error_file)) {
            open(FH, $error_file) or die;
            my @data = <FH>;        
            $app->print(@data);
        } else {
            return 'File Not Found.';
        }
    } else {
        my $login = $app->login;
        if (defined $login) {
            return &_auth($app,$path);
        } else {
            $app->{plugin_template_path} = 'tmpl';
            my $tmpl = 'login.tmpl';
            my %param;
            $param{script_url} = $ENV{"SCRIPT_NAME"};
            if ($return_url) {
                $param{return_url} = $return_url;
            } else {
                $param{return_url} = $org;
            }
            $app->set_header('Status','401 Unauthorized');
            return $app->build_page($tmpl, \%param);
        }
	}
}

sub _auth {
	my $app = shift;
	my $path = shift;
	my $file = $path.$suffix;
	unless  (-f $file) {
	    if ($file =~ /\/$/) {
	        my @idx_names = split(/,/, $indexes);
            foreach my $name (@idx_names) {
                my $cname = $file.$name;
                if (-f $cname) {
                    $file = $cname;
                    last;
                }
            }
	    }
	}
    if (-f $file) {
        my @stat = stat($file);
        my $modified_on = localtime($stat[9]);
        $modified_on = &str2time($modified_on);
        my $m_date = &time2str($modified_on);
        my $size = $stat[7];
#       $app->set_header('status', 200);
#       my $http_if_mod = $ENV{'HTTP_IF_MODIFIED_SINCE'};
#       $http_if_mod = &str2time($http_if_mod);
        $app->set_header('Last-Modified', $m_date);
#       if ($http_if_mod && ($http_if_mod >= $modified_on)) {
#           $app->set_header('Status','304 Not Modified');
#           $app->send_http_header();
#       } else {
        my $content_type = $1 if $path =~ /\.(gif|jpe?g|png)$/i;
	    if ($content_type) {
	        if ($content_type eq 'jpg') {
                $content_type = 'jpeg';
            }
	        $content_type = lc($content_type);
		    $content_type = 'image/'.$content_type;
		    $app->set_header('Content-Type', $content_type);
		}
        $app->set_header('Content-Length', $size);
        $app->send_http_header();
        open(FH, $file) or die;
        my @data = <FH>;        
        $app->print(@data);
#       }
    } else {
        $app->set_header('Status','404 File Not Found');
        $app->send_http_header();
        if (($error_file) && (-f $error_file)) {
            open(FH, $error_file) or die;
            my @data = <FH>;        
            $app->print(@data);
        } else {
            return 'File Not Found.';
        }
    }
}

1;
